unit uHidroConBombeo;

interface

uses
  Classes,
  Math, xMatDefs, MatReal, uSimplex, ucosa, uGlobs, uFechas, uNodos,
  SysUtils,
  uRandDispos,
  uEstados, uodt_types,
  uFichasLPD,
  uGeneradores,
  uconstantesSimSEE,
  uFuentesAleatorias, uCosaConNombre, uevapUruguay,
  uHidroConEmbalse,
  compol;

resourcestring
  rsHidroelectricaBombeo = 'Hidroeléctrica con bombeo';

type
  THidroConBombeo = class; // forward declaration

  { TFichaHidroConBombeo }

  TFichaHidroConBombeo = class(TFichaHidroConEmbalse)
  public
    // Pagos a los generadores
    PagoPorDisponibilidad_USD_MWh: NReal; // [USD/u-h] Pago por Potencia
    PagoPorEnergia_USD_MWh: NReal; // [USD/MWh] Pago por Potencia

    PMaxBombeo: NReal;
    QMaxBombeo: NReal;
    renBombeo: NReal;
    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      hmin_, hmax_: NReal; PuntosCotaVolumen_h, PuntosCotaVolumen_V: TDAOfNReal;
      AreaCuenca_ha: NReal; hDescarga: NReal; central_lagoDescarga: TGeneradorHidraulico;
      centralesAguasArriba: TListaCentralesAguasArriba;
      caQE, cbQE, ren_, Pmax_Gen_, Qmax_Turb_: NReal; fDispo_: NReal;
      tRepHoras: NReal; filtracion_Ca, filtracion_Cb, QaMuySeco: NReal;
      cotaMV0, cotaMV1, QMV1, QE_ControlDeCrecidaAPleno: NReal;
      HayRestriccionEmaxPasoDeTiempo: boolean; EmaxPasoDeTiempo: NReal;
      HayRestriccionQTmin: boolean; QTmin: NReal;
      PuntosControlCrecidaPorCotaYAportes_h: TDAOfNReal;
      PuntosControlCrecidaPorCotaYAportes_QA: TDAOfNReal;
      flg_ControlCrecidaPOrCotaYAportes: boolean;

      ImponerQminPorPoste: boolean;

      ControlarCotaPorDebajoDeObjetivo_sim, ControlarCotaPorEnciaDeObjetivo_sim: boolean;
      ControlarCotaPorDebajoDeObjetivo_opt, ControlarCotaPorEnciaDeObjetivo_opt: boolean;
      flg_ValorAguaExacto_hObjetivo:boolean;

      hObjetivo: NReal; Delta_cvaUSD_Hm3ParaControlDeCota: NReal;
      cv_USD_Hm3_ValorizadoManual: NReal; tomarCotaDeLaFuente: boolean;
      fuenteCota: TFuenteAleatoria; borneCota: string; saltoMinimoOperativo: NReal;
      cotaControlCrecida_Inicio: NReal; cotaControlCrecida_Pleno: NReal;
      PMaxBombeo: NReal; QMaxBombeo: NReal; renBombeo: NReal;
      flg_CalcularEvaporacionDelLago: boolean; flg_CalcularFiltracionDelLago: boolean;
      PagoPorDisponibilidad_USD_uh: NReal; PagoPorEnergia: NReal);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: integer=-2): TDataColumnListOfCosa; override;
  end;

  THidroConBombeo = class(THidroConEmbalse)
  public
    jPBombeado: integer; // coluna del simplex donde comienzan las PBombeado

    PBombeado: TDAOfNReal; // potencias de bombeo por poste
    ce_Bombeo: NReal; // coeficiente energético del bombeo
    QBombeado: NReal; // m3/s
    VBombeado: NReal; // hm3

    PMaxParaQMaxBombeo: NReal;
    // potencia correspondienta al QMaxBombeo con el salto dado
    PMaxBombeoDelPaso: NReal;
    // el mínimo entre la potencia de la bomba y el valor anterior
    cv_USD_MWh_Inc: NReal;
    ro_g_inv_renBombeo: NReal;

    procedure PrepararMemoria(globs: TGlobs); override;
    class function DescClase: string; override;
    procedure Sim_Cronica_Inicio; override;
    procedure PrepararPaso_ps; override;
    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_cargue(s: TSimplex); override;
    procedure CargarAportesAFilaCentralAguasAbajo(iresDestino: integer;
    // fila en la que cargar los aportes
      coefLlegada: NReal; // coeficiente de llegada por el que multiplicar los aportes
      cv_agua_entregada: NReal;
    // valor del agua a considerar en la central aguas abajo.
      s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;
{$IFDEF SPXCONLOG}
    procedure spx_NombrarVariables(s: TSimplex); override;
{$ENDIF}
    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    procedure Free; override;
    procedure CambioFichaPD; override;
    procedure PubliVars; override;
  end;

procedure THidroConBombeo_cambioFichaPD(Actor: TCosa);
procedure AlInicio;
procedure AlFinal;


var
  cnt_dbg: integer;


implementation

uses uActores;

procedure THidroConBombeo.prepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);
  setlength(PBombeado, globs.NPostes);
end;


class function THidroConBombeo.DescClase: string;
begin
  Result := rsHidroelectricaBombeo;
end;


procedure THidroConBombeo.prepararPaso_ps;
var
  Q_maxBombeo: double;
  cv_aux: double;
begin
  inherited prepararPaso_ps;

  // calculamos el bombeo máximo para que no desborde.

  {$IFDEF CONCONTROLCRECIDA}
  Q_maxBombeo := max(0, 0.99 * (min(V_ControlCrecida_Inicio, self.Vmax) -
    Vs_SinErogado - VAportePropio) / (globs.SegundosDelPaso / 1.0E6));
  {$ELSE}
  Q_maxBombeo := max(0, 0.99 * (V_hmax - Vs_SinErogado - VAportePropio) /
    (globs.SegundosDelPaso / 1.0E6));
  {$ENDIF}
  Q_maxBombeo := min(Q_maxBombeo, TFichaHidroConBombeo(pa).QMaxBombeo);
  ce_Bombeo := salto_actual * ro_g_inv_renBombeo;
  PmaxParaQMaxBombeo := Q_maxBombeo * ce_Bombeo * 3600;
  PMaxBombeoDelPaso := min(TFichaHidroConBombeo(pa).PMaxBombeo, PMaxParaQMaxBombeo);



  // NO puede pasar que estemos dispuesto a pagar más por el agua
  // bombeada que lo que estemos dispuesto a cobrar por el agua turbinada
  // esto tiene que ver con la concavidad de la función de costo futuro.
  if (cv_agua_USD_Hm3_Inc > cv_agua_USD_Hm3_Dec) then
  begin
    cv_aux := (cv_agua_USD_Hm3_Dec + cv_agua_USD_Hm3_Inc) / 2.0;
    cv_agua_USD_Hm3_Dec := cv_aux;
    cv_agua_USD_Hm3_Inc := cv_aux;

    cv_USD_MWh_Inc := cv_aux / (1.0E6 * ce_Bombeo);
    cv_USD_MWh := cv_aux * auxParaCalculos_cv;
  end
  else
    cv_USD_MWh_Inc := cv_agua_USD_Hm3_Inc / (1.0E6 * ce_Bombeo);

end;

procedure THidroConBombeo.opt_nvers(var ivar, ivae, ires: integer);
begin
  inherited opt_nvers(ivar, ivae, ires);
  jPBombeado := ivar;
  ivar := ivar + globs.NPostes; // las potencias de bombeo en cada poste
end;


procedure THidroConBombeo.opt_cargue(s: TSimplex);
var
  iposte: integer;
  jres: integer;
  invCEPorUnoESeis: NReal;

begin
  inherited opt_cargue(s);

  invCEPorUnoESeis := 1 / (ce_Bombeo * 1.0E6);

  // demanda del bombeo sobre la restricción de nodo
  // los coeficientes son 1 pues las potencias son negativas.
  for iposte := 0 to globs.NPostes - 1 do
    s.pon_e(nodo.ires + iposte, jPBombeado + iposte, 1);

  // valorización del agua bombeada
  for iposte := 0 to globs.NPostes - 1 do
    s.pon_e(s.nf, jPBombeado + iposte, -cv_USD_MWh_Inc * globs.durpos[iposte]);

  jres := ires;
  // restriccion de volumen >= 0
  for iposte := 0 to globs.NPostes - 1 do
    s.pon_e(jres, jPBombeado + iposte, -globs.durpos[iposte] * invCEPorUnoESeis);


  // si la central cargó restricción de energía máxima en el paso la salteo
  if (PuedoGenerar) and pA.HayRestriccionEmaxPasoDeTiempo then
    Inc(jres);

  // si es necesario agrego el bombeo a las restricciones de erogado mínimo.
  // no parece lógico que si hay erogado mínimo se admita bombear, pero lo escribo
  // igual
  if AplicarRestriccionDeErogadoMinimo then
  begin
    Inc(jres);
    if pa.ImponerQminPorPoste then
    begin
      for iposte := 0 to globs.NPostes - 1 do
        s.pon_e(jres + iposte, jPBombeado + iposte, globs.durpos[iposte] *
          invCEPorUnoESeis); // bombeo
    end
    else
      for iposte := 0 to globs.NPostes - 1 do
        s.pon_e(jres, jPBombeado + iposte, globs.durpos[iposte] * invCEPorUnoESeis);
    // bombeo
  end;
end;


procedure THidroConBombeo.CargarAportesAFilaCentralAguasAbajo(iresDestino: integer;
  // fila en la que cargar los aportes
  coefLlegada: NReal; // coeficiente de llegada por el que multiplicar los aportes
  cv_agua_entregada: NReal; // valor del agua a considerar en la central aguas abajo.
  s: TSimplex);

var
  menosCofAporte: NReal;
  flgValorizarEntrega: boolean;
  iposte: integer;
  icof: integer;
  valor: NReal;
begin
  inherited CargarAportesAFilaCentralAguasAbajo(
    iresDestino, coefLLegada, cv_agua_entregada, s);

  flgValorizarEntrega := cv_agua_entregada > 0;
  menosCofAporte := -coefLlegada; // esto es para tener en cuenta que en la
  // restricción de donde sacamos los volumens los mismos son extracciones y
  // en la que los sumamos son aportes.

  for iposte := 0 to globs.NPostes - 1 do
  begin
    icof := jPBombeado + iposte;
    valor := s.e(ires, icof) * menosCofAporte;
    s.acum_e(iresDestino, icof, valor); // Vk+1 >= 0
    if flgValorizarEntrega then
      s.acum_e(s.nf, icof, valor * cv_agua_entregada);
  end;
end;


procedure THidroConBombeo.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  iposte: integer;
begin
  inherited opt_FijarRestriccionesDeCaja(s);
  for iposte := 0 to globs.NPostes - 1 do
  begin
    s.cota_inf_set(jPBombeado + iposte, -self.PMaxBombeoDelPaso);
    s.cota_sup_set(jPBombeado + iposte, 0);
  end;
end;

procedure THidroConBombeo.Sim_Cronica_Inicio;
begin
  inherited Sim_Cronica_Inicio;
  QBombeado := 0;
  VBombeado := 0;
end;

procedure THidroConBombeo.opt_leerSolucion(s: TSimplex);
var
  iposte: integer;
begin
  Inc(cnt_dbg);
  inherited opt_leerSolucion(s);

  EnergiaConsumida := 0;
  for iposte := 0 to globs.NPostes - 1 do
  begin
    PBombeado[iposte] := s.xval(jPBombeado + iposte);
    EnergiaConsumida := EnergiaConsumida - PBombeado[iposte] * globs.durpos[iposte];

    if (PBombeado[iposte] < -0.1) and (P[iposte] > 0.1) then
      raise Exception.Create(
        'THidroConBombeo erogando y bombeando a la vez!!!, cnt_dbg: ' +
        IntToStr(cnt_dbg));
  end;

  VBombeado := EnergiaConsumida / (ce_Bombeo * 1.0E6);
  QBombeado := VBombeado * (1.0E6 * globs.invSegundosDelPaso);
end;


{$IFDEF SPXCONLOG}
procedure THidroConBombeo.spx_NombrarVariables(s: TSimplex);
var
  iposte, jres: integer;
begin
  inherited spx_NombrarVariables(s);
  for iposte := 0 to globs.NPostes - 1 do
    s.set_NombreVar(jPBombeado + iposte, Nombre + '_PBombeado[MW]' +
      IntToStr(iposte + 1));
end;

{$ENDIF}

function THidroConBombeo.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if ivar < jPBombeado then
    Result := inherited getNombreVar(ivar, nombre)
  else
  begin
    nombre := self.Nombre + '_PBombeado[MW]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end;
end;

procedure THidroConBombeo.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableVR('PBombeado', '[MW]', 6, 1, PBombeado, True, True);
  PublicarVariableNR('QBombeado', '[m3/s]', 8, 1, QBombeado, True);
  PublicarVariableNR('cva_dec', '[USD/MWh]', 8, 1, self.cv_USD_MWh, True);
  PublicarVariableNR('cva_inc', '[USD/MWh]', 8, 1, self.cv_USD_MWh_inc, True);
end;

procedure THidroConBombeo.Free;
begin
  setlength(PBombeado, 0);
  inherited Free;
end;


procedure THidroConBombeo.cambioFichaPD;
begin
  inherited cambioFichaPD;
  ro_g_inv_renBombeo := ro_Agua * g_gravitacion / TFichaHidroConBombeo(pA).renBombeo /
    (1.0E6 * 3600.0);
end;

{*********************************
*Métodos de TFichaHidroConBombeo*
*********************************}

constructor TFichaHidroConBombeo.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; hmin_, hmax_: NReal;
  PuntosCotaVolumen_h, PuntosCotaVolumen_V: TDAOfNReal; AreaCuenca_ha: NReal;
  hDescarga: NReal; central_lagoDescarga: TGeneradorHidraulico;
  centralesAguasArriba: TListaCentralesAguasArriba;
  caQE, cbQE, ren_, Pmax_Gen_, Qmax_Turb_: NReal; fDispo_: NReal;
  tRepHoras: NReal; filtracion_Ca, filtracion_Cb, QaMuySeco: NReal;
  cotaMV0, cotaMV1, QMV1, QE_ControlDeCrecidaAPleno: NReal;
  HayRestriccionEmaxPasoDeTiempo: boolean; EmaxPasoDeTiempo: NReal;
  HayRestriccionQTmin: boolean; QTmin: NReal;
  PuntosControlCrecidaPorCotaYAportes_h: TDAOfNReal;
  PuntosControlCrecidaPorCotaYAportes_QA: TDAOfNReal;
  flg_ControlCrecidaPOrCotaYAportes: boolean;

  ImponerQminPorPoste: boolean;
  ControlarCotaPorDebajoDeObjetivo_sim, ControlarCotaPorEnciaDeObjetivo_sim: boolean;
  ControlarCotaPorDebajoDeObjetivo_opt, ControlarCotaPorEnciaDeObjetivo_opt: boolean;
  flg_ValorAguaExacto_hObjetivo:boolean;
  hObjetivo: NReal; Delta_cvaUSD_Hm3ParaControlDeCota: NReal;
  cv_USD_Hm3_ValorizadoManual: NReal; tomarCotaDeLaFuente: boolean;
  fuenteCota: TFuenteAleatoria; borneCota: string; saltoMinimoOperativo: NReal;
  cotaControlCrecida_Inicio: NReal; cotaControlCrecida_Pleno: NReal;
  PMaxBombeo: NReal; QMaxBombeo: NReal; renBombeo: NReal;
  flg_CalcularEvaporacionDelLago: boolean; flg_CalcularFiltracionDelLago: boolean;
  PagoPorDisponibilidad_USD_uh: NReal; PagoPorEnergia: NReal);
begin
  inherited Create(
    capa,
    fecha, periodicidad,
    hmin_, hmax_,
    PuntosCotaVolumen_h, PuntosCotaVolumen_V,
    AreaCuenca_ha,
    hDescarga,
    central_lagoDescarga,
    centralesAguasArriba,
    caQE, cbQE, ren_, Pmax_Gen_, Qmax_Turb_,
    fDispo_, tRepHoras,
    filtracion_Ca, filtracion_Cb, QaMuySeco,
    cotaMV0, cotaMV1, QMV1, QE_ControlDeCrecidaAPleno,
    HayRestriccionEmaxPasoDeTiempo,
    EmaxPasoDeTiempo,
    HayRestriccionQTmin,
    QTmin,

    PuntosControlCrecidaPorCotaYAportes_h,
    PuntosControlCrecidaPorCotaYAportes_QA,
    flg_ControlCrecidaPOrCotaYAportes,

    ImponerQminPorPoste,
    ControlarCotaPorDebajoDeObjetivo_sim,
    ControlarCotaPorEnciaDeObjetivo_sim,
    ControlarCotaPorDebajoDeObjetivo_opt,
    ControlarCotaPorEnciaDeObjetivo_opt,
    flg_ValorAguaExacto_hObjetivo,
    hObjetivo,
    Delta_cvaUSD_Hm3ParaControlDeCota,
    cv_USD_Hm3_ValorizadoManual,
    tomarCotaDeLaFuente,
    fuenteCota,
    borneCota,
    saltoMinimoOperativo,
    cotaControlCrecida_Inicio,
    cotaControlCrecida_Pleno,
    flg_CalcularEvaporacionDelLago,
    flg_CalcularFiltracionDelLago,
    PagoPorDisponibilidad_USD_uh, PagoPorEnergia);

  self.PMaxBombeo := PMaxBombeo;
  self.QMaxBombeo := QMaxBombeo;
  self.renBombeo := renBombeo;
end;

procedure TFichaHidroConBombeo.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('PMaxBombeo', PMaxBombeo);
  f.wr('QMaxBombeo', QMaxBombeo);
  f.wr('renBombeo', renBombeo);
end;

class function TFichaHidroConBombeo.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
var
   list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TFichaHidroConBombeo, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('PMaxBombeo', TFichaHidroConBombeo(nil).PMaxBombeo, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('QMaxBombeo',TFichaHidroConBombeo(nil).QMaxBombeo, TDataTypeDoubleOfCosa, 113, -1);
  list.AddDataColumn('renBombeo',TFichaHidroConBombeo(nil).renBombeo, TDataTypeDoubleOfCosa, 113, -1);
  result:=list;
end;


constructor TFichaHidroConBombeo.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('PMaxBombeo', PMaxBombeo);
  f.rd('QMaxBombeo', QMaxBombeo);
  f.rd('renBombeo', renBombeo);
  f.EjecutarLectura;
end;

procedure THidroConBombeo_cambioFichaPD(Actor: TCosa);
begin
  (Actor as THidroConBombeo).cambioFichaPD;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(THidroConBombeo.ClassName, THidroConBombeo);
  registrarClaseDeCosa(TFichaHidroConBombeo.ClassName, TFichaHidroConBombeo);
end;

procedure AlFinal;
begin
end;

begin
  cnt_dbg := 0;
end.
