(* En esta unidad se implementa el actor TDemandaDetallada
básicamente una demanda detallada está descripta por los valores de la demanda
para cada hora y para cada día en el horizonte de tiempo.

Los datos de la demanda son leidos de un archivo binario que tiene
al inicio un record con : el primer día de datos indicado como un TDateTime
y la fecha del día siguiente al último día de datos.

Luego en el binario hay un recor para cada día con los 24 valores correspondientes
a las potencias horarias en MW.

*)(*Se agrega la capacidad de cargar al simplex requerimientos de reserva
desde el 8/5/15 por Micho mvarela@adme.com.uy*)
unit udemandadetallada;

  {$MODE Delphi}

interface

uses
  SysUtils,
  uauxiliares,
  xmatdefs,
  uglobs, uDemandas, unodos, usimplex,
  ufechas,
  ucosa, udatoshorariosdetallados,
  uconstantesSimSEE, uFuentesAleatorias, Classes, contnrs, uFichasLPD;

resourcestring
  rs_DemandaDetallada = 'Demanda Detallada';
  rs_DemandaDetalladaComienzaEn = 'Demanda detallada, comienza en';
  rs_LaSimulacionTieneFechaDeInicio = 'la simulación tiene fecha de inicio';
  rs_LaOptimizacionTieneFechaDeInicio = 'la optimización tiene fecha de inicio';
  rs_DemandaDetalladaTerminaEn = 'Demanda detallada, termina en';
  rs_LaSimulacionTieneFechaDeFin = 'la simulación tiene fecha de fin';
  rs_LaOptimizacionTieneFechaDeFin = 'la optimización tiene fecha de fin';

type

  { TDemandaDetallada }

  TDemandaDetallada = class(TDemanda)
  public
    ArchiDatos: TArchiRef;
    datos: TDatosHorariosDetallados;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD; nodo: TNodo; falla_profundidad, falla_costo: TDAOfNReal; ArchiDatos: string; fuente: TFuenteAleatoria; nombreBorne: string; SumarEnergiaHr, flg_SumarParaPostizado: boolean; icf_Fuente: TFuenteAleatoria; icf_NombreBorne: string; fReserva: Nreal);

    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;

    procedure PrepararMemoria(globs: TGlobs); override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    function PotenciaFirme: NReal; override;

    //      procedure prepararSim; override;
    procedure prepararPaso_as; override;

    class function CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: integer = -2): TDataColumnListOfCosa; override;

    procedure Free; override;
    {===Se agregan con la reserva===}
    procedure opt_cargue(s: TSimplex); override;
    {=======Micho===================}
  private
    CacheActivo: boolean;
  end;

(* Escalones de falla por defecto
( Uruguay-invierno 2006 )
escf [pu]  0.05  0.075  0.075  0.8
cvf [USD/MWh]  140  400  1200  2000
*)

(* Deforma una curva de potencia horaria aplicando factores de crecimiento de
energía (fce) y del pico (fcp).
Por ejemplo, si el pico decrese 10% y la energía debe aumentar 5% los factores
serán fce= 1.05 y fcp= 0.9

El resultado es TRUE si todas las potencias resultantes son positivas y false
si alguna quedó negativa.
*)

procedure AlInicio;
procedure AlFinal;

implementation

(* Escalones de falla por defecto
( Uruguay-invierno 2006 )
escf [pu]  0.05  0.075  0.075  0.8
cvf [USD/MWh]  140  400  1200  2000
*)

//-----------------------------
// Métodos de TDemandaDetallada
//=============================

constructor TDemandaDetallada.Create(capa: integer; nombre: string; nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD; nodo: TNodo; falla_profundidad, falla_costo: TDAOfNReal; ArchiDatos: string; fuente: TFuenteAleatoria; nombreBorne: string; SumarEnergiaHr, flg_SumarParaPostizado: boolean; icf_Fuente: TFuenteAleatoria; icf_NombreBorne: string; fReserva: NReal);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, nodo,
    falla_profundidad, falla_costo,
    fuente, nombreBorne,
    icf_Fuente, icf_NombreBorne, SumarEnergiaHr, flg_SumarParaPostizado);
  datos := nil;

  if FileExists(ArchiDatos) then
    self.ArchiDatos := TArchiRef.Create(ArchiDatos)
  else
    raise ExceptionFileNotFound.Create(ArchiDatos, 'Creando el actor ' + ClaseNombre);

  CacheActivo := False;
  Self.fReserva := fReserva;
end;

constructor TDemandaDetallada.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.rdArchRef('ArchiDatos', archiDatos);
  f.EjecutarLectura;
  CacheActivo := False;
  if not ArchiDatos.testear then
    raise ExceptionFileNotFound.Create(ArchiDatos.archi, 'Creando el actor ' +
      ClaseNombre);
end;

procedure TDemandaDetallada.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrArchRef('ArchiDatos', ArchiDatos);
end;

procedure TDemandaDetallada.PrepararMemoria(globs: TGlobs);
begin
  inherited prepararMemoria(globs);

  if datos = nil then
    datos := TDatosHorariosDetallados.Create(archiDatos.archi);

  if globs.EstadoDeLaSala <> CES_SIN_PREPARAR then
  begin
    if (globs.fechaIniSim.AsDt < datos.fechaPrimerDia) then
      raise Exception.Create(rs_DemandaDetalladaComienzaEn + ': ' +
        DateTimeToStr(datos.FechaPrimerDia) +
        ' ' + rs_LaSimulacionTieneFechaDeInicio + ': ' + globs.fechaIniSim.AsStr);
    if (globs.fechaIniOpt.AsDt < datos.fechaPrimerDia) then
      raise Exception.Create(rs_DemandaDetalladaComienzaEn + ': ' +
        DateTimeToStr(datos.FechaPrimerDia) + ' ' +
        rs_LaOptimizacionTieneFechaDeInicio + ': ' + globs.fechaIniOpt.AsStr);

    if globs.fechaFinSim.asDt >= (datos.fechaUltimodia + 1) then
      raise Exception.Create(rs_DemandaDetalladaTerminaEn + ': ' +
        DateTimeToStr(datos.FechaUltimoDia + 23.0 / 24.0) + ' ' +
        rs_LaSimulacionTieneFechaDeFin + ': ' + globs.fechaFinSim.AsStr);
    if globs.fechaFinOpt.asDt >= (datos.fechaUltimodia + 1) then
      raise Exception.Create(rs_DemandaDetalladaTerminaEn + ': ' +
        DateTimeToStr(datos.FechaUltimoDia + 23.0 / 24.0) + ' ' +
        rs_LaOptimizacionTieneFechaDeFin + ': ' + globs.fechaFinOpt.AsStr);
  end;
end;

function TDemandaDetallada.InfoAd_: string;
begin
  Result := ArchiDatos.archi;
end;

class function TDemandaDetallada.DescClase: string;
begin
  Result := rs_DemandaDetallada;
end;

function TDemandaDetallada.PotenciaFirme: NReal;
begin
  Result := paUnidades.nUnidades_Operativas[0] * vmax(self.PPa);
end;


procedure TDemandaDetallada.prepararPaso_as;
begin
  if globs.SalaMinutal then
  begin
    PHoraria[0] := datos.interpol(globs.FechaInicioDelPaso.AsDt);
  end
  else
    datos.ReadBuff_horario(PHoraria, globs.FechaInicioDelpaso.AsDt);
  inherited prepararPaso_as;
end;


procedure TDemandaDetallada.Free;
begin
  if ArchiDatos <> nil then
  begin
    ArchiDatos.Free;
    ArchiDatos := nil;
  end;

  if datos <> nil then
    datos.Free;
  inherited Free;
end;

class function TDemandaDetallada.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TDemandaDetallada);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list := inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  list.AddDataColumn('ArchiDatos', TDemandaDetallada(nil).ArchiDatos, TDataTypeArchiRefOfCosa, 113, -1); //TArchiRef
  Result := list;
end;

{===Lo podría agregar si quiero agregar un despacho de falla de reserva==}{
procedure TDemandaDetallada.opt_nvers(var ivar, ivae, ires: integer);
begin
end;
 }

procedure TDemandaDetallada.opt_cargue(s: TSimplex);

var
  iposte, iescalon: integer;
  ibaseres: integer;
begin
  inherited opt_cargue(s);
  if (globs.flg_ReservaRotante) then
  begin
    {$IFDEF SPXCONLOG}
    spx_NombrarVariables(s);
    {$ENDIF}
    ibaseres := nodo.ires;
    // Carga el término constante de las restricciones de demanda
    for iposte := 0 to High(PPa) do
      s.acum_e(ibaseres + Length(PPA) + iposte, s.nc, -PPa[iposte] * fReserva);
  end;
end;


procedure AlInicio;
begin
  registrarClaseDeCosa(TDemandaDetallada.ClassName, TDemandaDetallada);
end;

procedure AlFinal;
begin
end;

end.
