unit ufechas;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  SysUtils, DateUtils, uauxiliares;

const
  HorasDelAnio = 365 * 24;
  DiasDeLosMeses: array [1..12] of integer =
    (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
  DiasDeLosMesesBisiesto: array [1..12] of integer =
    (31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

//rch@20140708 ... bug??  un_Minuto = 1.0 / ( 24.0 / 60.0 );
  dt_un_Minuto = 1/(24.0*60.0);
  dt_medio_Minuto = dt_un_Minuto / 2.0;

  dt_un_Segundo = dt_un_Minuto / 60.0;
  dt_medio_Segundo = dt_un_Segundo / 2.0;

  dt_treintaSegundos = 30.0 * dt_un_Segundo;

  horaToDt  = 1.0 / 24.0;
  dtToHora  = 24.0;
// bisiestos son los mltiplos de 4 que no son mltiplos de 100 salvo que se mltiplo de 400
// entonces, en 400 aos hay 3 centurias con 24 bisiestos y una con 25
// la cantidad de bisiestos en 400 aos es entonces 3 * 24 + 25 = 25*4-3 = 97
  DiasDelAnioMedio = ( 97.0 * 366.0 + ( 400.0 - 97.0 ) * 365 ) / 400.0;  //=365.2425 considerando 97 bisiestos cada 400 aos.
  DiasDelMesMedio = DiasDelAnioMedio/ 12.0;
  HorasSemana52 = DiasDelAnioMedio * 24.0 / 52.0;

  dt_CicloAnual = DiasDelAnioMedio;
  dt_CicloDiario = 24.0;

type
  TFecha = class
  public
    dt: TDateTime;
    function DiasDelAnio: integer;
    // function DiasDelMes(fecha : TFecha) : Integer;

    constructor Create_Dt(dt: TDateTime);
    constructor Create_Str(const fecha: string);
    constructor Create_ISOStr(const fecha: string);
    constructor Create_Any_Str(const fecha: string);
    class function Create_Clone(fecha: TFecha): TFecha;
    constructor Create_OffsetHoras(fecha: TFecha; horasDesplazamiento: double );
    constructor Create_OffsetDT(fecha: TFecha; dtDesplazamiento: TDateTime);
    constructor Create_AnioSemana53(anio, semana53: integer);
    constructor Create_AnioMesDia(anio, mes, dia: integer);

    procedure addHoras(horas: integer);
    procedure addAnios(anios: integer);
    procedure addMeses(meses: integer);
    procedure addDias(dias: integer);

    function HorasHasta(fecha: TFecha): integer;

    function diasDesdeElInicioDelAnio: TDateTime;
    function dtPrimerDiaDelAnio: TDateTime;

    function horasDesdeElInicioDelAnio: integer;
    function aniosHasta(fecha: TFecha): integer;

    // 1 si Self> fecha, 0 si Sefl=fecha y -1 si self < fecha
    // compara con presicin de 1 minuto.
    function EsMayorQue(fecha: TFecha): integer;

    function EsMayorQueEnElAnio(fecha: TFecha): integer;
    function entre(fechaIni, fechaFin: TFecha): boolean;

    // esta funcin recibe un dos fechas que supone ordenadas fechaIni < fechaFin
    // y que la distancia entre fechas es inferior a un ao.
    // compara se Self pertenece al cajn definido por [fechaIni, fechaFin)
    // desplazando Self la cantidad de aos que sea necesario.
    function entreSinConsiderarAnio(fechaIni, fechaFin: TFecha): boolean;
    procedure PonerIgualA(fecha: TFecha); overload;
    procedure PonerIgualA(const fecha: string); overload;
    //palfaro@110128_0402
    //Le cambie el nombre a esta funcin porque hacen cosas distintas,
    //PonerIgualAMasOffsetHoras suma el argumento / 24 y la otra lo suma
    //directamente, el llambado poda resultar erroneo si se quera sumar el
    //offset en horas pero la funcin se llamaba con un valor real
    procedure PonerIgualAMasOffsetHoras(fecha: TFecha; offsetEnHoras: integer);
    procedure PonerIgualAMasOffsetDT(fecha: TFecha; offsetDt: TDateTime);
    function enAnioBisiesto: boolean;

    //Comparadores binarios para facilitar la lectura de las condiciones
    function igualQue_DT(dt: TDateTime): boolean;
    function mayorQue_DT(dt: TDateTime): boolean;

    function igualQue(fecha: TFecha): boolean;
    function menorQue(fecha: TFecha): boolean;
    function menorOIgualQue(fecha: TFecha): boolean;
    function mayorQue(fecha: TFecha): boolean;
    function mayorOIgualQue(fecha: TFecha): boolean;

    procedure setDt(const fecha: string);
    function getAnio: integer;
    procedure SetAnio(const anio: integer);
    function getMes: integer;
    function getSemana52: integer;
    procedure SetSemana52(const semana52: integer);
    //Retorna el primer da del anio
    function getDia: integer;
    procedure SetDia(const dia: integer);
    function getHora: integer;
    procedure setHora(const hora: integer);

    function AsStr: string;
    function AsISOStr(): string;
    function AsDt: TDateTime;



    function TipoDeDia: integer; // 0 habil, 1 Sabado, 2 Domingo
    // procedure free;

    property S: string Read asStr;
    //Para debug, permite ver la fecha en formato string en el inspector
    property anio: integer Read getAnio Write setAnio;
    property mes: integer Read getMes;
    property semana52: integer Read getSemana52 Write setSemana52;
    property dia: integer Read getDia Write setDia;
    property hora: integer Read getHora Write setHora;
  end;

  PFecha     = ^TFecha;
  TDAofFecha = array of TFecha;

(* ojo la semana puede ser de 7 das y entonces hay hasta semana 53
o la semana puede ser calculada para que en el ao haya 52 semanas
para evitar confuciones donde aparezca la semana pongamos 53 o 52 segn
el rango que pueda tener la variable semana y as evitamos lios (o los atenuamos)*)
function AnioSemana53ToDateTime(anio, semana53: word): TDateTime;
procedure DateTimeToAnioSemana53(dt: TDateTime; var anio, semana53: word);

function IsoStrToDateTime(const fecha: string): TDateTime;
function DateTimeToIsoStr(const fecha: TDateTime): string;

function minFecha(fecha1, fecha2: TFecha): TFecha;
function maxFecha(fecha1, fecha2: TFecha): TFecha;
//Retorna true si (fecha >= fecha1) y (fecha < fecha2)
function fechaEntre(fecha, fecha1, fecha2: TDateTime): boolean;
function horasHasta(fechaDesde, fechaHasta: TDateTime): integer;


// usar esta en lugar de la de SysUtils
// la de Sysutils pierde las horas si no hay minutos  ej: '1/1/2015 06'
// lo intepreta como '1/1/2015'
function StrToDateTime( Str_DT: String ): TDateTime;

implementation


function StrToDateTime(Str_dt: String): TDateTime;
var
  sDate, sTime: string;
  i: integer;
begin
  sDate:= trim( Str_dt );
  i:= pos( ' ', sDate );
  if i > 0 then
  begin
    sTime:= copy( sDate, i+1, length( sDate ) - i );
    delete( sDate, i, length( sDate ) - i + 1 );
    result:= StrToDate( sDate ) + StrToTime( sTime );
  end
  else
    result:= sysutils.StrToDateTime(Str_dt);
end;

function AnioSemana53ToDateTime(anio, semana53: word): TDateTime;
var
  primerDiaDelAnio: TDateTime;
  dd: double;
begin
  primerDiaDelAnio := EncodeDate(anio, 1, 1);
  dd     := (semana53 - 1) * 7.0;
  Result := dd + primerDiaDelAnio;
end;

procedure DateTimeToAnioSemana53(dt: TDateTime; var anio, semana53: word);
var
  Year, Month, Day: word;
  primerDiaDelAnio: TDateTime;
  dd: integer;
begin
  DecodeDate(dt, Year, Month, Day);
  primerDiaDelAnio := EncodeDate(Year, 1, 1);
  dd     := trunc(dt - primerDiaDelAnio);
  Anio   := Year;
  semana53 := (dd div 7) + 1;
end;


function NextInt(var s: string; const sep: string): integer;
var
  i: integer;
  r: string;
begin
  i := pos(sep, s);
  if i > 0 then
  begin
    r := copy(s, 1, i - 1);
    Delete(s, 1, i);
  end
  else
  begin
    r := s;
    s := '';
  end;
  if r = '' then
    Result := 0
  else
    Result := StrToInt(r);
end;


function AnyStrToDateTime(const fecha: string): TDateTime;
var
  anio, mes, dia: integer;
  hora, min: integer;
  s:   string;
  sep: string;
begin
  s := fecha;
  if pos('/', s) > 0 then
  begin
    sep  := '/';
    dia  := nextInt(s, ' ');
    mes  := nextInt(s, sep);
    anio := nextInt(s, sep);
  end
  else
  begin
    sep  := '-';
    anio := nextInt(s, sep);
    mes  := nextInt(s, sep);
    dia  := nextInt(s, ' ');
  end;
  if pos(':', s) > 0 then
    hora := nextInt(s, ':')
  else
    hora := 0;
  if pos(':', s) > 0 then
    min := nextInt(s, ':')
  else
    min := 0;

  Result := EncodeDateTime(anio, mes, dia, hora, min, 0, 0);
end;

function IsoStrToDateTime(const fecha: string): TDateTime;
var
  anio, mes, dia: integer;
  hora, min, seg: integer;
//   0000000001111111111
//   1234567890123456789
//  'aaaa-mm-dd hh:mm:ss'
begin
(*
  writeln( '*0000000001111111111*' );
  writeln( '*1234567890123456789*' );
  writeln( '*aaaa-mm-dd hh:mm:ss*' );
  writeln( '*'+fecha+'*' );
*)
  anio := StrToInt(copy(fecha, 1, 4));
  mes  := StrToInt(copy(fecha, 6, 2));
  dia  := StrToInt(copy(fecha, 9, 2));
  if anio = 0 then
  begin
    anio := 1900;
    if mes = 0 then
    begin
      mes:= 1;
      if dia = 0 then dia:= 1;
    end;
  end;

  hora:= 0;
  min:= 0;
  seg:= 0;

  if length(fecha) >= 13 then
  begin
    hora   := StrToInt(copy(fecha, 12, 2));
    if length( fecha ) >= 16 then
    begin
      min    := StrToInt(copy(fecha, 15, 2));
      if length( fecha ) >= 19 then
        seg    := StrToInt(copy(fecha, 18, 2))
      else
        seg:= 0;
    end
    else
      min:= 0;
  end
  else
    hora := 0;


  Result := EncodeDateTime(anio, mes, dia, hora, min, seg, 0);
end;

function pad2d(s: string): string;
var
  res: string;
begin
  res := s;
  while length(res) < 2 do
    res  := '0' + res;
  Result := res;
end;

function DateTimeToIsoStr(const fecha: TDateTime): string;
var
  Year, Month, Day: word;
  Hour, Min, Sec, MSec: word;
  res: string;
begin
  DecodeDate(fecha, Year, Month, Day);
  DecodeTime(fecha, Hour, Min, Sec, MSec);
  res := IntToStr(Year) + '-' + pad2d(IntToStr(Month)) + '-' + pad2d(IntToStr(Day));

  if (Hour + Min) > 0 then
    res  := res + ' ' + pad2d(IntToStr(Hour)) + ':' + pad2d(IntToStr(Min));
  Result := res;
end;

function minFecha(fecha1, fecha2: TFecha): TFecha;
begin
  if fecha1.dt <= fecha2.dt then
    Result := fecha1
  else
    Result := fecha2;
end;

function maxFecha(fecha1, fecha2: TFecha): TFecha;
begin
  if fecha1.dt >= fecha2.dt then
    Result := fecha1
  else
    Result := fecha2;
end;

//Retorna true si (fecha >= fecha1) y (fecha < fecha2)
function fechaEntre(fecha, fecha1, fecha2: TDateTime): boolean;
begin
  Result := (fecha >= fecha1) and (fecha < fecha2);
end;

function horasHasta(fechaDesde, fechaHasta: TDateTime): integer;
begin
  Result := trunc(((fechaHasta - fechaDesde) * 24) + 0.1);
end;

constructor TFecha.Create_Dt(dt: TDateTime);
begin
  inherited Create;
  self.dt := dt;
end;

constructor TFecha.Create_Str(const fecha: string);
var
  tdc: char;
  sDate, sTime: string;
  i: integer;

begin
  inherited Create;
  if fecha <> '0' then
    try
      self.dt:= strToDateTime( fecha );
        except
      tdc := DefaultFormatSettings.DateSeparator;
      case tdc of
        '/': DefaultFormatSettings.DateSeparator := '-';
        '-': DefaultFormatSettings.DateSeparator := '/';
      end;

      try
        self.dt := StrToDateTime(fecha);
      finally
        DefaultFormatSettings.DateSeparator := tdc;
      end;
    end
  else
    self.Create_Dt(0);
end;

constructor TFecha.Create_ISOStr(const fecha: string);
begin
  inherited Create;
  self.dt := IsoStrToDateTime(fecha);
end;

constructor TFecha.Create_Any_Str(const fecha: string);
begin
  inherited Create;
  self.dt := AnyStrToDateTime(fecha);
end;

class function TFecha.Create_Clone(fecha: TFecha): TFecha;
begin
  if fecha <> nil then
    Result := TFecha.Create_Dt(fecha.dt)
  else
    Result := nil;
end;

constructor TFecha.Create_OffsetHoras(fecha: TFecha; horasDesplazamiento: double );
begin
  inherited Create;
  self.dt := fecha.dt + horasDesplazamiento * horaToDt;
end;

constructor TFecha.Create_OffsetDT(fecha: TFecha; dtDesplazamiento: TDateTime);
begin
  inherited Create;
  self.dt := fecha.dt + dtDesplazamiento;
end;

constructor TFecha.Create_AnioSemana53(anio, semana53: integer);
begin
  Create_Dt( AnioSemana53ToDateTime(anio, semana53));
end;

constructor TFecha.Create_AnioMesDia(anio, mes, dia: integer);
begin
  inherited Create;
  Self.dt := EncodeDate(anio, mes, dia);
end;

procedure TFecha.addHoras(horas: integer);
begin
  dt := dt + horas * horaToDt;
end;

procedure TFecha.addAnios(anios: integer);
begin
  dt := IncYear(dt, anios);
end;

procedure TFecha.addMeses(meses: integer);
begin
  dt := IncMonth(dt, meses);
end;

procedure TFecha.addDias(dias: integer);
begin
  dt := dt + dias;
end;

function TFecha.HorasHasta(fecha: TFecha): integer;
begin
  Result := trunc(((fecha.dt - dt) * 24) + 0.1);
end;

function TFecha.diasDesdeElInicioDelAnio: TDateTime;
var
  anio, mes, dia:     word;
  dtPrimerDiaDelAnio: TDateTime;
begin
  DecodeDate(dt, anio, mes, dia);
  dtPrimerDiaDelAnio := EncodeDate(anio, 1, 1);
  Result := (dt - dtPrimerDiaDelAnio);
end;


function TFecha.dtPrimerDiaDelAnio: TDateTime;
var
  anio, mes, dia:     word;
  dtPrimerDiaDelAnio: TDateTime;
begin
  DecodeDate(dt, anio, mes, dia);
  result := EncodeDate(anio, 1, 1);
end;

function TFecha.horasDesdeElInicioDelAnio: integer;
var
  anio, mes, dia:     word;
  dtPrimerDiaDelAnio: TDateTime;
begin
  DecodeDate(dt, anio, mes, dia);
  dtPrimerDiaDelAnio := EncodeDate(anio, 1, 1);
  Result := trunc((dt - dtPrimerDiaDelAnio) * 24 + 0.1);
end;

function TFecha.aniosHasta(fecha: TFecha): integer;
var
  anioSelf, anioFecha, mes, dia: word;
begin
  DecodeDate(dt, anioSelf, mes, dia);
  DecodeDate(fecha.dt, anioFecha, mes, dia);
  Result := anioFecha - anioSelf;
end;

function TFecha.EsMayorQue(fecha: TFecha): integer;
var
  minutos: integer;
begin
  minutos := trunc((self.dt - fecha.dt) * (24 * 60));
  if minutos > 0 then
    Result := 1
  else if minutos < 0 then
    Result := -1
  else
    Result := 0;

  (*
  if self.dt < fecha.dt then result := -1
  else if self.dt = fecha.dt then result := 0
  else result := 1
  *)

end;

function TFecha.EsMayorQueEnElAnio(fecha: TFecha): integer;
var
  anioSelf, anioFecha, mesSelf, mesFecha, diaSelf, diaFecha: word;
begin
  DecodeDate(dt, anioSelf, mesSelf, diaSelf);
  DecodeDate(fecha.dt, anioFecha, mesFecha, diaFecha);
  if mesSelf < mesFecha then
    Result := -1
  else if mesSelf = mesFecha then
  begin
    if diaSelf < diaFecha then
      Result := -1
    else if diaSelf = diaFecha then
      Result := 0
    else
      Result := 1;
  end
  else
    Result := 1;
end;

function TFecha.entre(fechaIni, fechaFin: TFecha): boolean;
begin
  Result := (self.dt >= fechaIni.dt) and (self.dt < fechaFin.dt);
  //  result:= (EsMayorQue(fechaIni) >= 0) and (esMayorQue(fechaFin) <= 0);
end;

function TFecha.entreSinConsiderarAnio(fechaIni, fechaFin: TFecha): boolean;
var
  anioSelf, mesSelf, diaSelf: word;
  anioIni, mesIni, diaIni:    word;
  anioFin, mesFin, diaFin:    word;

  function ordenados(m1, d1, m2, d2: integer): boolean;
  begin
    Result := (m1 < m2) or ((m1 = m2) and (d1 <= d2));
  end;

  function ordenados_estricto(m1, d1, m2, d2: integer): boolean;
  begin
    Result := (m1 < m2) or ((m1 = m2) and (d1 <= d2));
  end;

begin
  DecodeDate(self.dt, anioSelf, mesSelf, diaSelf);
  DecodeDate(fechaIni.dt, anioIni, mesIni, diaIni);
  DecodeDate(fechaFin.dt, anioFin, mesFin, diaFin);


  if ordenados_estricto(mesIni, diaIni, mesFin, diaFin) then
  begin // todo en orden comparo sencillo
    Result := ordenados(mesIni, diaIni, mesSelf, diaSelf) and
      ordenados_estricto(mesSelf, diaSelf, mesFin, diaFin);
  end
  else
  begin  // el cajn pasa el lmite del ao comparo "abierto"
    Result := ordenados(mesIni, diaIni, mesSelf, diaSelf) or
      ordenados_estricto(mesSelf, diaSelf, mesFin, diaFin);
  end;

  (**** esto parece un bolazo
  if (mesIni < mesSelf) or                              //si fechaIni es menor o
     ((mesIni = mesSelf) and (diaIni <= diaSelf)) then  //igual en el anio que fecha
  begin
    if (mesFin > mesSelf) or                            //si fechaFin es mayor en
       ((mesFin = mesSelf) and (diaFin > diaSelf)) or   //el anio que fecha
       (anioIni < anioFin) then                         //o fechaFin es en un anio anterior anioIni
      result:= true
    else
      result:= false;
  end
  else if ((anioIni < anioFin) and (mesFin > mesSelf) or ((mesFin = mesSelf) and (diaFin > diaSelf))) then
  begin
    result:= true;
  end
  else
    result:= false;
    *****)
end;

procedure TFecha.PonerIgualA(fecha: TFecha);
begin
  dt := fecha.dt;
end;

procedure TFecha.PonerIgualA(const fecha: string);
var
  tdc: char;
begin
  if (fecha <> '0') and (fecha <> 'Auto') and (fecha <> 'auto') and (fecha <> 'AUTO') then
    try
      self.dt := StrToDateTime(fecha);
    except
      tdc := SysUtils.DefaultFormatSettings.DateSeparator;
      case tdc of
        '/': DefaultFormatSettings.DateSeparator := '-';
        '-': DefaultFormatSettings.DateSeparator := '/';
      end;

      try
        self.dt := StrToDateTime(fecha);
      finally
        DefaultFormatSettings.DateSeparator := tdc;
      end;
    end
  else
    self.Create_Dt(0);
end;

procedure TFecha.PonerIgualAMasOffsetHoras(fecha: TFecha; offsetEnHoras: integer);
begin
  dt := fecha.dt + offsetEnHoras * horaToDt;
end;

procedure TFecha.PonerIgualAMasOffsetDT(fecha: TFecha; offsetDt: TDateTime);
begin
  dt := fecha.dt + offsetDt;
end;

function TFecha.enAnioBisiesto: boolean;
var
  anio, mes, dia: word;
begin
  DecodeDate(dt, anio, mes, dia);
  Result := IsLeapYear(anio);
end;

function TFecha.igualQue_DT(dt: TDateTime): boolean;
begin
  Result := Self.dt = dt;
end;

function TFecha.mayorQue_DT(dt: TDateTime): boolean;
begin
  Result := self.dt > dt;
end;

function TFecha.igualQue(fecha: TFecha): boolean;
begin
  Result := self.dt = fecha.dt;
end;

function TFecha.menorQue(fecha: TFecha): boolean;
begin
  Result := self.dt < fecha.dt;
end;

function TFecha.menorOIgualQue(fecha: TFecha): boolean;
begin
  Result := self.dt <= fecha.dt;
end;

function TFecha.mayorQue(fecha: TFecha): boolean;
begin
  Result := self.dt > fecha.dt;
end;

function TFecha.mayorOIgualQue(fecha: TFecha): boolean;
begin
  Result := self.dt >= fecha.dt;
end;

function TFecha.getAnio: integer;
var
  Year, Month, Day: word;
begin
  DecodeDate(dt, Year, Month, Day);
  Result := Year;
end;

function TFecha.getMes: integer;
var
  Year, Month, Day: word;
begin
  DecodeDate(dt, Year, Month, Day);
  Result := month;
end;

procedure TFecha.setAnio(const Anio: integer);
var
  Year, Month, Day: word;
begin
  DecodeDate(dt, Year, Month, Day);
  Year := Anio;
  if (month = 2) and ( day = 29 ) and not IsLeapYear(anio) then
    dt   := EncodeDate(Year, 3, 1)
  else
    dt   := EncodeDate(Year, Month, Day)
end;

function TFecha.getSemana52: integer;
var
  Year, Month, Day: word;
  primerDiaDelAnio: TDateTime;
begin
  DecodeDate(dt, Year, Month, Day);
  primerDiaDelAnio := EncodeDate(Year, 1, 1);
  if self.enAnioBisiesto then
    Result := trunc((dt - primerDiaDelAnio) / (366.0/52.0)) + 1
  else
    Result := trunc((dt - primerDiaDelAnio) /  (365.0/52.0))  + 1
end;

procedure TFecha.setSemana52(const semana52: integer);
var
  Year, Month, Day: word;
  primerDiaDelAnio: TDateTime;
  dtDias: TDateTime;
begin
  DecodeDate(dt, Year, Month, Day);
  primerDiaDelAnio := EncodeDate(Year, 1, 1);
  if self.enAnioBisiesto then
    dtDias := (366.0/52.0) * (semana52-1 )
  else
    dtDias := (365.0/52.0) * ( semana52-1 );

  dt := primerDiaDelAnio + dtDias;
end;


function TFecha.getDia: integer;
var
  Year, Month, Day: word;
  primerDiaDelAnio: TDateTime;
begin
  DecodeDate(dt, Year, Month, Day);
  primerDiaDelAnio := EncodeDate(Year, 1, 1);
  Result := trunc(dt - primerDiaDelAnio) + 1;
end;

procedure TFecha.setDia(const dia: integer);
var
  Year, Month, Day: word;
  primerDiaDelAnio: TDateTime;
begin
  DecodeDate(dt, Year, Month, Day);
  primerDiaDelAnio := EncodeDate(Year, 1, 1);

  dt := primerDiaDelAnio + (dia - 1);
end;

function TFecha.getHora: integer;
begin
  Result := trunc(frac(dt) * 24 + dt_medio_Segundo) mod 24;
end;

procedure TFecha.setDt(const fecha: string);
begin
  if pos('/', fecha) <> 0 then
    self.dt := StrToDateTime(fecha)
  else
    self.dt := IsoStrToDateTime(fecha);
end;


procedure TFecha.setHora(const hora: integer);
begin
  dt := trunc(dt) + hora * horaToDt;
end;

function TFecha.AsStr(): string;
begin
  Result := DateTimeToStr(dt);
end;

function TFecha.AsISOStr(): string;
begin
  Result := DateTimeToIsoStr(dt);
end;

function TFecha.AsDt: TDateTime;
begin
  Result := dt;
end;

function TFecha.TipoDeDia: integer; // 0 habil, 1 Sabado, 2 Domingo
var
  dow: integer;
begin
  dow := SysUtils.DayOfWeek(dt);
  if dow = 1 then
    Result := 2
  else
  if dow = 7 then
    Result := 1
  else
    Result := 0;
end;

{procedure TFecha.free;
begin
  inherited free;
end;}

function TFecha.DiasDelAnio: integer;
var
  anio, mes, dia: word;
begin
  DecodeDate(dt, anio, mes, dia);
  if not IsLeapYear(anio) then
    Result := 365
  else
    Result := 366;
end;

{function TFecha.DiasDelMes(fecha : TFecha) : Integer;
begin
  if (fecha.anio mod 4 <> 0) or (fecha.anio mod 100 = 0) then
    result := DiasDeLosMeses[fecha.mes]
  else
    result := DiasDeLosMesesBisiesto[fecha.mes];
end;}

end.

